// (c)2004 Paul Forgey
#ifndef KRELLHARDWAREDEVICE_H
#define KRELLHARDWAREDEVICE_H

#include "../syCommon/Device.h"
#include "KrellProperties.h"
#include "KrellModule.h"
#include "../../Interfaces/KrellInterface/KrellStatus.h"

#include <map>
#include <string>

class CProperty;

class CKrellHardwareDevice : public CDevice
{
    public:
        typedef void (CKrellHardwareDevice::*HandlerProc)(CPropertyValue *);
    
        typedef std::map< std::string, CProperty * > propertymap_type;
        typedef std::map< const CProperty *, HandlerProc > handlermap_type;

    public:
        CKrellHardwareDevice (DEVICE_HANDLE, const DeviceModule *, 
                              bool enabled, const std::string &deviceName, 
                              const std::string &deviceDescription,
                              int model);
        virtual ~CKrellHardwareDevice ();
        
        virtual bool DeviceInit (void);
        virtual bool DeviceRun (HANDLE terminate, HANDLE changeRequest);
        
        virtual void ButtonPressNotification (int buttonID);
        virtual bool SubscriptionNotification (const std::string &listName, 
                const std::string &filter, const CDataPack *);
        
        void Command (const char *string);

        inline const KrellModule *GetDeviceModule (void) const
            { return static_cast< const KrellModule * > (
                    CDevice::GetDeviceModule ()); }
        
    private:
        /* We don't mark these properties as not allowing same value change.  
           For example, setting trigger2 to On is perfectly legitimate to 
           re-send the command. However, we sure don't want to re-set it
           same value back again as the result of updating the property from
           live status.
         */
        bool changePropertyValue (CProperty *property, 
                const std::string &newValue, bool allowSameValue);
    
        void audioInputChanged (CProperty *, BYTE);
        void trimChanged (CProperty *, BYTE);
        void balanceChanged (BYTE); // zone1 only, also compression
        void modeLEDChanged (BYTE);
        void decodeChanged (BYTE);
        void inputChanged (CProperty *, BYTE);
        void eqChanged (BYTE); // also zone2 balance
        void statusChanged (BYTE, BYTE);
        void streamChanged (BYTE);
        void videoInputChanged (BYTE);
        void volumeChanged (CProperty *, BYTE);
        
        void changeProperty (CPropertyValue *);
        void changePower (CPropertyValue *);
        void changeDeviceSelect (CPropertyValue *);
        void changeVolumeLevel (CPropertyValue *);
        void changeTrim (CPropertyValue *);
        void changeBalance (CPropertyValue *);
        void changeCompression (CPropertyValue *);
        void changeRoomEQ (CPropertyValue *);
        void changeDirectAudio (CPropertyValue *);
        void changeDirectVideo (CPropertyValue *);
        void changeTrigger (CPropertyValue *);
        void changeAudioDecode (CPropertyValue *);
        void changeMuting (CPropertyValue *);
        void changeButton (CPropertyValue *);
        
    protected:
        int m_model;
        
        KrellStatus m_status, m_priorStatus;
        propertymap_type m_properties;
        handlermap_type m_handlers;
};

#endif /* KRELLHARDWAREDEVICE_H */
