// Copyright (c)2004 Paul Forgey 
#ifndef MEEDIO_DEVICEMODULE_H
#define MEEDIO_DEVICEMODULE_H

#include "DeviceAPI.h"

#include <vector>
#include <string>
#include <map>

class CDevice;

class DeviceModule
{
public:
    typedef CDevice *(*CreateDeviceProc)(intptr_t userData, DEVICE_HANDLE, 
            const DeviceModule *, bool isEnabled,
            const std::string &name, const std::string &description);

    struct ModuleDeviceInfo {
        DeviceInfo          deviceInfo;
        CreateDeviceProc    createProc;
        intptr_t            createProcData;
        std::string         name;
    };
    typedef std::vector< ModuleDeviceInfo > devices_type;
    typedef std::map< DEVICE_HANDLE, CDevice * > instances_type;
    typedef std::vector< char * > strings_type;

protected:
    devices_type        m_devices;
    instances_type      m_instances;
    
private:
    HANDLE              m_dllInstance;
    DeviceCallBackInfo  m_cbi;
    ModuleInfo          m_moduleInfo;
    strings_type        m_strings;    

private:
    CDevice *instanceFromHandle (DEVICE_HANDLE);
    const CDevice *instanceFromHandle (DEVICE_HANDLE) const;

protected:
    const char *dupCString (const std::string &);

public:
    DeviceModule (HANDLE dllInstance, const DeviceCallBackInfo *, 
            const std::string &version, const std::string &name);
    virtual ~DeviceModule ();

    virtual bool initializeModule (void) =0;
    virtual bool registerProperties (int deviceNumber) =0;

    int addDevice (
            intptr_t context, 
            CreateDeviceProc createProc,
            const std::string &deviceName, 
            eDeviceType deviceType,
            const std::string &interfaceSignatures, 
            eDeviceFlags deviceFlags,
            const std::string &wizardDescription, 
            const std::string &wizardFinalText);

    inline const ModuleInfo *moduleInfo (void) const 
        { return &m_moduleInfo; }
        
    inline HANDLE dllInstance (void) const
        { return m_dllInstance; }

    inline const DeviceCallBackInfo *callBackInfo (void) const
        { return &m_cbi; }
        
    const DeviceInfo *deviceInfo (int deviceNumber) const;

    bool initialize (DEVICE_HANDLE, bool isEnabled, const std::string &name, 
            const std::string &description);
            
    bool setEnabled (DEVICE_HANDLE, bool isEnabled);
    
    bool run (DEVICE_HANDLE, HANDLE terminateEvent, 
                             HANDLE propertyChangeEvent);

    bool buttonPress (DEVICE_HANDLE, int buttonID);
    
    bool subscriptionNotification (DEVICE_HANDLE, const std::string &list,
            const std::string &filter, const CDataPack *pack);

    bool registerProperty (
            int deviceNumber, 
            const std::string &name,
            const std::string &description,
            bool isDefault,
            const std::string &initialValue,
            eIOType ioType,
            bool isPersistent,
            bool allowSameValueChange,
            bool trace,
            bool isDynamic);
            
    bool createProperty (
            const std::string &name,
            const std::string &description,
            ePropertyType type);
            
    bool createPropertyValue (
            const std::string &name,
            const char *value);

    inline bool createPropertyValue (
            const std::string &name,
            const std::string &value)
        { return createPropertyValue (name, value.c_str ()); }
            
    template< typename InputIterator >
    bool createAndRegisterProperty (
            int deviceNumber,
            const std::string &name,
            const std::string &description,
            bool isDefault,
            const std::string &initialValue,
            eIOType ioType,
            bool isPersistent,
            bool allowSameValueChange,
            bool trace,
            bool isDynamic,
            ePropertyType type,
            InputIterator firstValue,
            InputIterator lastValue)
    {
        if (!registerProperty (deviceNumber, name, description, 
                isDefault, initialValue, ioType, isPersistent, 
                allowSameValueChange, trace, isDynamic))
        {
            if (!createProperty (name, description, type))
                return false;
            
            while (firstValue != lastValue)
                createPropertyValue (name, *(firstValue++));
                
            return registerProperty (deviceNumber, name, description, 
                    isDefault, initialValue, ioType, isPersistent,
                    allowSameValueChange, trace, isDynamic);
        }
        return true;
    }


};

#endif /* MEEDIO_DEVICEMODULE_H */
