// ====================================================================
// Copyright (c) 2004 Paul Forgey
// original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part as a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File          : KrellInstance.h
// Origin Date   : 11/05/2004
// derived from TemplateInstance.h 09/7/2002
// Purpose       : This file contains the implementation of the
//                   CKrellInstance class
// ====================================================================

#pragma once

#include "../syCommon/HardwareInstance.h"
#include "../../syCommon/InterfaceAPI.h"

#include "KrellStatus.h"
#include "CommThread.h"

#include <string>

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// CKrellInstance - Sample Hardware Interface Plugin
//       instance class.  This class is derived from CHardwareInstance
//       to inherit all helper functions.
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
class CKrellInstance : public CHardwareInstance
{
public:
   CKrellInstance (HARDWARE_INSTANCE_HANDLE hInstance, const CallBackInfo*);
   virtual ~CKrellInstance ();

   //
   // Derived from virtual methods in CHardwareInstance class to service
   // standard server requests.
   bool RegisterProperties (void);
   bool UpdatePropertyValue (const std::string &name, const std::string &value);
   bool PropertyButtonClick( int nButtonCallbackID );
   bool Query (const std::string &query, void *pResponse);
   void Terminate (void);

   bool Enable (bool);
   bool SetComPort (const std::string &port);
   void UpdateStatus (const KrellStatus *);

private:
    const char *GetCodeForPropertyValue (const char *propertyValue, 
            const char * const *list) const;

    bool OpenComPort (void);
    void CloseComPort (void);

    // this device uses no handshaking, and what we sent are very small bursts of 
    // data and not very often, so if it blocks, something is seriously wrong
    bool SendData (const char *string);

public:
   //
   // Plugin specific interfaces.
   HARDWARE_RC Command (const InterfaceArgumentPack* pPack);

private:
    bool            m_bEnabled;
    std::string     m_serialName;
    HANDLE          m_serialDevice;
    KrellStatus     m_status;
    OVERLAPPED      m_writeOV;
    CommThread      m_statusReader;    
    
};

