// ====================================================================
// Original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           HardwareInstance.h
// Origin Date:    9/4/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains the class definition for CHardwareInterface.
//    The CHardwareInterface class is a helper class for writing HouseBot
//    Hardware plugin Interfaces.  The class wraps the server callback
//    function pointers for easier calling.
// ====================================================================

#ifndef HARDWAREINSTANCE_H
#define HARDWAREINSTANCE_H

#include <string>
#include <vector>

#include "HardwareAPI.h"

//
// Forward definitions
class CDataPack;

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// CHardwareInstance
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
class CHardwareInstance
{
public:
    virtual ~CHardwareInstance();

    //
    // Derived classes have the option of deriving these methods.  These
    // methods map to exports that are called by the server.
    virtual bool Init (void);
    virtual void Terminate (void);
    virtual bool RegisterProperties (void);
    virtual bool UpdatePropertyValue (const std::string &name, const std::string &value);
    virtual bool PropertyButtonClick (int buttonId);
    virtual bool Query (const std::string &query, void *);

    inline HARDWARE_INSTANCE_HANDLE GetInstanceHandle (void) const 
        { return m_instance; }
   
    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////
    // Call back helper wrappers.  See HardwareAPI.h for description
    // of the methods and their parameters.
    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////
    bool RegisterHardwareModuleProperty (const std::string &propertyName, 
            P_TYPE pType, const char** aValues, 
            const std::string &defaultValue, bool showInMainConfiguration,
            const std::string &wizardText );

    bool SetPropertyLimits (const std::string &propertyName, int lowLimit, 
            int highLimit);

    bool RegisterHardwareModuleButton (int buttonId, 
            const std::string &buttonHeading, const std::string &buttonText);

    std::string GetPropertyNameFromHandle (PROPERTY_VALUE_HANDLE) const;
    std::string GetPropertyValueFromName (const std::string &name) const;
    std::string GetPropertyValueFromName (PROPERTY_VALUE_HANDLE) const;
    std::string GetPropertyValueFromHandle (PROPERTY_VALUE_HANDLE) const;    

    bool GetPropertyInfoAtIndex (int index, std::string *propertyName,
            std::string *propertyValue) const; 
    
    bool SetPropertyValueFromName (const std::string &propertyName, 
            const std::string &propertyValue);
    bool SetPropertyNameAndValueAtIndex (int index, 
            const std::string &propertyName, const std::string &propertyValue);
    
    bool DeletePropertyValueByNameAndValue (const std::string &propertyName, 
            const std::string &propertyValue);

    bool DeletePropertyValueByIndex (int index);
    
    void NotifySubscribedDevices (const std::string &listName, 
            const std::string &filter, const CDataPack *pDataPack );
    
    bool GetIRCodes (std::vector< IRCode > *codes) const;
    bool GetIRCodeFromPosition (unsigned int position, IRCode* code) const;
    bool AddOrUpdateIRCode (IRCode *code, bool newCode);
    bool RemoveIRCode (const IRCode *code);

    // const correct modify
    inline BOOL UpdateIRCode (const IRCode *code) 
        { return AddOrUpdateIRCode (const_cast< IRCode * > (code), FALSE); }
    
    bool IsRemoteControlDescriptionUnique (
            const std::string &remoteDescription) const;
    bool GetRemoteControlList (std::vector< RemoteControl > *remotes) const; 
    bool AddRemoteControl (const std::string &remoteDescription, 
            int nFrequency, const std::string &manufacturer, 
            const std::string &model);
                        
    void  TraceMessage (TRC_TYPE ttType, const char* szMessage, ... ) const;


    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////
    // Other helper methods
    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    std::string CharToDump (const unsigned char *data, size_t len) const;
    std::string GetLastErrorAsString() const;

    int DoesDeviceExist (const std::string &deviceName, bool exactMatch, 
        const std::string &deviceDescription, const std::string &propertyName,
        const std::string &propertyValue) const;

    int CreateDevice (const std::string &name, const std::string &description,
        const std::string &comment, HARDWARE_INSTANCE_HANDLE); 

    bool SetDeviceProperty (int deviceId, const std::string &propertyName, 
        const std::string &propertyValue);

protected:
    CHardwareInstance (HARDWARE_INSTANCE_HANDLE, const CallBackInfo *);

    bool CheckForServerVersion (int nMajor, int nMinor, int nSubMinor) const;


    HARDWARE_INSTANCE_HANDLE    m_instance;
    CallBackInfo                m_cbi;
};


// ====================================================================
// Class Name  : CMFCStateSwitcher
// Description : This class helps to manage the MFC thread/module
//                state of MFC DLLs.  It is used by the CHardwareInstance
//                class before calling back into the server.
// ====================================================================
#ifdef __AFX_H__
class CMFCStateSwitcher
{
public:
   CMFCStateSwitcher() { m_pDLLModuleState = AfxSetModuleState( AfxGetThreadState()->m_pPrevModuleState );  }
   ~CMFCStateSwitcher() { AfxSetModuleState( m_pDLLModuleState ); }

   AFX_MODULE_STATE*    m_pDLLModuleState;
};

#else
class CMFCStateSwitcher
{
public:
   CMFCStateSwitcher() {};
};

#endif

#endif /* HARDWAREINSTANCE_H */
