// ====================================================================
// Original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.

#ifndef DATAPACK_H
#define DATAPACK_H

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// CDataPack
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
class CDataPack
{
public:
    static const int kMaxValues = 50;

public:
    //
    // Basic constructor and destructor
    CDataPack();
    ~CDataPack(); // NO VTBL!!

    // ====================================================================
    // Method Name : AddData - For adding string data
    // Parameters  : szDataName - Name of data to add.
    // 				: szDataValue - String data value
    // Description : This method will add a named string data element to the pack.
    // Returns     : TRUE if added successfully.  FALSE on error
    // ====================================================================
    bool AddData (const char *dataName, const char *dataValue);

    // ====================================================================
    // Method Name : AddData - For adding binary data
    // Parameters  : szDataName - Name of data to add.
    // 				: pDataValue - Data value
    // 				: nDataSize - Size of Data in pDataValue
    // Description : This method will add a named binary data element to the pack.
    // Returns     : TRUE if added successfully.  FALSE on error
    // ====================================================================
    bool AddData (const char *dataName, const void *data, int dataSize);

    // ====================================================================
    // Method Name : GetData - For reading string data
    // Parameters  : szDataName - Name of data to return value.
    // Description : This method will return the string data that corresponds to the data name specified
    // Returns     : Value of data specified by szDataName.  "" If szDataName is invalid.
    // ====================================================================
    const char *GetData  (const char *dataName) const;

    // ====================================================================
    // Method Name : GetBinaryData - For reading binary data
    // Parameters  : szDataName - Name of data to return value.
    // 				: nDataSize - Reference that contains the size of the data returned.
    // Description : This method will return the binary data that corresponds to the data name specified
    // Returns     : Value of data specified by szDataName.  NULL If szDataName is invalid.
    // ====================================================================
    const void *GetBinaryData (const char *dataName, int *dataSize) const;

private:
    const char *const *findData (const char *dataName) const;

private:
    // need to preserve binary compatability
    char*       m_dataNames[kMaxValues];
    BYTE*       m_dataValues[kMaxValues];
    int		    m_dataLens[kMaxValues];		// Only valid for binary types.
};

#endif /* DATAPACK_H */
