// Copyright (c)2004 Paul Forgey
#include "StdAfx.h"
#include "InterfaceAPI.h"

#include <algorithm>

InterfaceArgument::InterfaceArgument ()
{
    m_type = kType_Unknown;
    m_value.userData = NULL;
}

InterfaceArgument::InterfaceArgument (PROPERTY_VALUE_HANDLE handle)
{
    propertyValueHandle (handle);
}

InterfaceArgument::InterfaceArgument (const char *string)
{
    stringValue (string);
}

InterfaceArgument::InterfaceArgument (int value)
{
    intValue (value);
}

InterfaceArgument::InterfaceArgument (const void *data)
{
    userData (data);
}

InterfaceArgument::~InterfaceArgument ()
{
}

void InterfaceArgument::propertyValueHandle (PROPERTY_VALUE_HANDLE handle)
{
    m_type = kType_PropertyValueHandle;
    m_value.propertyValueHandle = handle;
}

PROPERTY_VALUE_HANDLE InterfaceArgument::propertyValueHandle (void) const
{
    if (m_type == kType_PropertyValueHandle)
        return m_value.propertyValueHandle;
    return NULL;
}

void InterfaceArgument::stringValue (const char *string)
{
    m_type = kType_String;
    m_value.stringValue = string;
}

const char *InterfaceArgument::stringValue (void) const
{
    if (m_type == kType_String)
        return m_value.stringValue;
    return NULL;
}

void InterfaceArgument::intValue (int value)
{
    m_type = kType_Int;
    m_value.intValue = value;
}

const int *InterfaceArgument::intValue (void) const
{
    if (m_type == kType_Int)
        return &m_value.intValue;
    return NULL;
}

void InterfaceArgument::userData (const void *data)
{
    m_type = kType_Void;
    m_value.userData = data;
}

const void *InterfaceArgument::userData (void) const
{
    if (m_type == kType_Void)
        return m_value.userData;
    return NULL;
}

InterfaceArgumentPack::InterfaceArgumentPack (const char *signature, int argsCount)
{
    m_version = kInterfacePackVersion;
    m_signature = signature;    
    m_argsCount = argsCount;
    m_args = new InterfaceArgument[argsCount];
}

InterfaceArgumentPack::~InterfaceArgumentPack ()
{
    delete[] m_args;
}

void InterfaceArgumentPack::resize (int newSize)
{
    if (newSize == m_argsCount)
        return;

    InterfaceArgument *newData = new InterfaceArgument[newSize];
    std::copy (&m_args[0], &m_args[std::min (m_argsCount, newSize)],
            &newData[0]);
    delete[] m_args;
    if (newSize > m_argsCount)
        std::fill (&newData[m_argsCount], &newData[newSize],
                InterfaceArgument ());
    m_argsCount = newSize;
    m_args = newData;
}
