// ====================================================================
// Original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           InterfaceAPI.h
// Origin Date:    9/4/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains common structures and enumerations that relate
//    to the InterfaceArgumentPack which is used to pass parameter data
//    from Devices to Hardware Interfaces.
// ====================================================================

#ifndef MEEDIO_INTERFACEAPI_H
#define MEEDIO_INTERFACEAPI_H

#include "HBTypes.h"

/* binary compatible with original InterfaceAPI structures */

class InterfaceArgument
{
public:
    enum ArgumentType {
        kType_Unknown,
        kType_PropertyValueHandle,
        kType_String,
        kType_Void,
        kType_Int
    };
    
private:
    ArgumentType m_type;
    union {
        PROPERTY_VALUE_HANDLE   propertyValueHandle;
        const char              *stringValue;
        int                     intValue;
        const void              *userData;
    } m_value;
    
public:
    InterfaceArgument ();
    InterfaceArgument (PROPERTY_VALUE_HANDLE);
    InterfaceArgument (const char *);
    InterfaceArgument (int);
    InterfaceArgument (const void *);

    ~InterfaceArgument (); // NO VTBL!
    
    inline ArgumentType type (void) const { return m_type; }
    
    // accessors all return NULL if the type isn't right
    PROPERTY_VALUE_HANDLE propertyValueHandle (void) const;
    void propertyValueHandle (PROPERTY_VALUE_HANDLE);
    
    const char *stringValue (void) const;
    void stringValue (const char *);
    
    const int *intValue (void) const;
    void intValue (int);
    
    const void *userData (void) const;
    void userData (const void *);
};

class InterfaceArgumentPack
{
public:
    static const int kInterfacePackVersion =1;

private:
    int                     m_version;
    const char              *m_signature;
    int                     m_argsCount;
    InterfaceArgument       *m_args;

public:
    InterfaceArgumentPack (const char *signature, int argsCount);
    ~InterfaceArgumentPack (); // NO VTBL!

    inline InterfaceArgument &operator [] (size_t index) 
        { return m_args[index]; }
    inline const InterfaceArgument &operator [] (size_t index) const
        { return m_args[index]; }
    
    inline bool correctVersion (void) const 
        { return m_version == kInterfacePackVersion; }
    inline int size (void) const 
        { return m_argsCount; }
    void resize (int);
};

#endif /* MEEDIO_INTERFACEAPI_H */
